<?php
/****
 * vB Optimise
 * Copyright 2010; Deceptor
 * All Rights Reserved
 * Code may not be copied, in whole or part without written permission
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
if (!is_object($vbulletin))
{
	die('Cannot access directly.');
}

define('vboptimise', true);

require_once(DIR . '/vboptimise/includes/class_operator_model.php');

class vb_optimise
{
	/**
	* Version info
	*
	* @public	mixed
	*/	
	public static $jQueryVersion 	= '1.7.2';
	public static $version 			= '2.5.0';
	public static $versionnumber	= '250';

	/**
	* Opcache Operator if assigned
	*
	* @public	object
	*/
	public static $cache = null;

	/**
	* Array of valid Opcache Operator models. Use register() to extend via hooks.
	*
	* @public	array
	*/
	public static $models = array(
		'none',
		'apc',
		'xcache',
		'memcache',
		'eaccelerator',
		'wincache',
		'filecache',
	);

	/**
	* Array of valid cacher libraries to be called through cache(). Use register() to extend via hooks.
	*
	* @public	array
	*/
	public static $cachers = array(
		'datastore',
		'templates',
		'style',
		'thanksstats',
		'usertagstats',
		
	);

	/**
	* Integer to hold current sessions MySQL queries saved.
	*
	* @public	integer
	*/
	public static $query_count = 0;

	/**
	* String to hold user-defined cache prefix
	*
	* @public	string
	*/
	public static $prefix = '';

	/**
	* Boolean value to determine if page output is cached
	*
	* @private	boolean
	*/
	private static $guestcache = false;

	/**
	* Escapes a string and makes it JavaScript-safe
	*
	* @param	mixed	The string or array to make JS-safe
	*/	
	public static function jsEscapeString(&$arr)
	{
		$find = array(
			"\r\n",
			"\n",
			"\t",
			'"'
		);
		
		$replace = array(
			'\r\n',
			'\n',
			'\t',
			'\"',
		);
		
		$arr = str_replace($find, $replace, $arr);
	}
	
	/**
	* Encodes a string as a JSON object (consistent behaviour instead of relying on PHP built-in functions)
	*
	* @param	mixed	The string or array to encode
	* @param	boolean	(Optional) Whether this is an associative array
	* @param	boolean	(Optional) Whether we should escape the string or if they have already been escaped
	*/	
	public static function encodeJSON($arr, $assoc = true, $doescape = true)
	{
		if ($doescape)
		{
			self::jsEscapeString($arr);
		}
		if (!$assoc)
		{
			// Not associative, simple return
			return '{"' . implode('","', $arr) . '"}';
		}
		
		$content = array();
		foreach ((array)$arr as $key => $val)
		{
			if (is_array($val))
			{
				// Recursion, definition: see recursion
				$val = self::encodeJSON($val);
				$content[] = '"' . $key . '":' . $val;
			}
			else
			{
				$content[] = '"' . $key . '":"' . $val . '"';
			}
		}
		
		return '{' . implode(',', $content) . '}';
	}
	
	/**
	* Outputs a JSON string to the browser 
	*
	* @param	mixed	array to output
	*/	
	public static function outputJSON($json, $full_shutdown = false)
	{
		if (!headers_sent())
		{
			// Set the header
			header('Content-type: application/json');
		}
		
		// Create JSON
		$json = self::encodeJSON($json);
		
		// Turn off debug output
		self::$vbulletin->debug = false;
		
		if (defined('VB_API') AND VB_API === true)
		{
			print_output($json);
		}

		//run any registered shutdown functions
		if (intval(self::$vbulletin->versionnumber) > 3)
		{
			$GLOBALS['vbulletin']->shutdown->shutdown();
		}
		if (defined('NOSHUTDOWNFUNC'))
		{
			if ($full_shutdown)
			{
				exec_shut_down();
			}
			else
			{
				self::$vbulletin->db->close();
			}
		}
		
		$sendHeader = false;
		switch(self::$vbulletin->options['ajaxheader'])
		{
			case 0 :
				$sendHeader = true;
				
			case 1 :
				$sendHeader = false;
				
			case 2 :
			default:
				$sendHeader = (strpos($_SERVER['SERVER_SOFTWARE'], 'Microsoft-IIS') !== false);
		}

		if ($sendHeader)
		{
			// this line is causing problems with mod_gzip/deflate, but is needed for some IIS setups
			@header('Content-Length: ' . strlen($json));
		}
		
		// Finally spit out JSON
		echo $json;
		die();
	}

	/**
	* Register extensions to valid Opcache Operators or Cacher Libraries via hooks.
	*
	* @param	string	Extension to register
	* @param	string	Item to register
	*/
	public static function register($to, $what = '')
	{
		if (trim($what) == '')
		{
			return fale;
		}

		if (!self::$$to || in_array($what, self::$$to))
		{
			return false;
		}

		self::$$to = array_merge(self::$$to, array($what));
		self::report('Registered ' . $what . ' to ' . $to);
	}

	/**
	* Reports internal message through to vBulletins debug combo-box.
	*
	* @param	string	Message to report
	*/
	public static function report($msg = '')
	{
		if (VB_AREA != 'AdminCP') // Dev debug messages mess with the vBulletin CMS ACP Management =/
		{
			devdebug('vBOptimise: ' . $msg);
		}
	}

	/**
	* Assigns and connects a valid Opcache Operator
	*
	* @param	string	Opcache Operator to assign
	*/
	public static function assign($model = '')
	{
		global $vbulletin;

		if (!$vbulletin->options['vbo_online'])
		{
			return false;
		}

		if (self::$cache !== null)
		{
			self::report('assign(' . $model . ') failed. vB Optimise already has a operator.');
			return false;
		}

		if (!in_array($model, self::$models))
		{
			trigger_error('vB Optimise could not assign the operator model \'' . $model . '\'. If you are requesting a custom operator please extend the $models array via vb_optimise::register().', E_USER_ERROR);
		}

		if (!file_exists(DIR . '/vboptimise_pro/includes/operators/' . $model . '.php'))
		{
			if (!file_exists(DIR . '/vboptimise/includes/operators/' . $model . '.php'))
			{
				self::report('Unable to assign Opcache Operator (' . $model . '). vB Optimise disabled.');
				$vbulletin->options['vbo_online'] = false;
				return false;
			}
			else
			{
				require_once(DIR . '/vboptimise/includes/operators/' . $model . '.php');
			}
		}
		else
		{
			require_once(DIR . '/vboptimise_pro/includes/operators/' . $model . '.php');
		}

		$class = 'vb_optimise_' . $model;

		self::$cache = new $class();

		if (self::$cache->connect())
		{
			self::report('Assigned Opcache Operator (' . $model . ')');
		}
		else
		{
			self::report('Unable to assign Opcache Operator (' . $model . '). vB Optimise disabled.');
			$vbulletin->options['vbo_online'] = false;
		}
	}

	/**
	* Checks if vB Optimise Cache can be used
	*
	* @param	string	Cache perm to check (optional)
	* 
	* @return	Boolean	True on success
	*/
	public static function check_cache($cache = '')
	{
		global $vbulletin;

		if (!$vbulletin->options['vbo_online'])
		{
			return false;
		}

		if ($cache != '' && !$vbulletin->options['vbo_cache_' . $cache])
		{
			return false;
		}

		if (self::$cache == null)
		{
			self::report('cache(' . $cache . ') failed. No assignment made yet.');
			return false;
		}

		return true;
	}

	/**
	* Executes a cacher library with optional arguments
	*
	* @param	string	Cache Library to execute
	* @param	mixed	Optional. Reference argument
	* @param	mixed	Optional. Reference argument
	*/
	public static function cache($what = '', &$argument = false, &$argumentb = false, &$argumentc = false)
	{
		global $vbulletin;

		if (!self::check_cache($what))
		{
			return false;
		}

		if (!in_array($what, self::$cachers))
		{
			trigger_error('vB Optimise could not assign the cacher routine \'' . $what . '\'. If you are requesting a custom routine please extend the $cachers array via vb_optimise::register().', E_USER_ERROR);
		}

		if (!file_exists(DIR . '/vboptimise_pro/includes/cachers/cacher_' . $what . '.php'))
		{
			if (file_exists(DIR . '/vboptimise/includes/cachers/cacher_' . $what . '.php'))
			{
				require_once(DIR . '/vboptimise/includes/cachers/cacher_' . $what . '.php');
			}
			else
			{
				self::report('Unable to assign cacher (' . $what . '). vB Optimise disabled.');
				$vbulletin->options['vbo_online'] = false;
				return false;
			}
		}
		else
		{
			require_once(DIR . '/vboptimise_pro/includes/cachers/cacher_' . $what . '.php');
		}
	}

	/**
	* Triggers automatic cache flushes within Admin CP based on executions.
	*/
	public static function update()
	{
		global $vbulletin;

		if (!$vbulletin->options['vbo_online'])
		{
			return false;
		}

		self::report('Looking for conditions to flush cache automatically...');

		// ACP Actions to flush cache on
		$actions = array(
			'updatetemplate',
			'inserttemplate',
			'productimport',
			'rebuild',
			'insertstyle',
			'update',
			'clear_cache',
			'purgecache',
			'displayorder',
			'savestylevar',
			'insert',
			'import',
			'replace',
			'kill',
			'productsave',
			'productdependancy',
			'productcode',
			'productdisable',
			'productenable',
			'productkill',
			'dooptions',
		);

		// ACP actions to kill whole cache as a failsafe
		$full = array(
			'clear_cache',
			'purgecache',
		);

		if (in_array($_REQUEST['do'], $actions))
		{
			if (in_array($_REQUEST['do'], $full))
			{
				self::$cache->full_flush();
			}
			else
			{
				self::$cache->flush();
			}

			self::report('Automatically flushed cache.');
		}
	}

	/**
	* Pushes statistical data to the cache temporarily
	*
	* @param	int	Number of queries saved.
	*/
	public static function stat($num = 0)
	{
		global $vbulletin;

		self::$query_count += $num;

		if (!$vbulletin->options['vbo_online'] || !$vbulletin->options['vbo_stat'])
		{
			return false;
		}

		if (self::$cache == null)
		{
			return false;
		}

		$stats = self::$cache->get('vb.optimiser.stats');
		$stats = intval($stats) + $num;

		if ($num > 0)
		{
			self::$cache->set('vb.optimiser.stats', $stats);
		}
		else
		{
			self::$cache->set('vb.optimiser.stats', 0);
		}
	}

	/**
	* Updates statistical database from cache
	*/
	public static function updatestats()
	{
		global $vbulletin;

		if (!$vbulletin->options['vbo_online'] || !$vbulletin->options['vbo_stat'])
		{
			return false;
		}

		require_once(DIR . '/includes/functions_misc.php');

		$date = $vbulletin->db->escape_string(gmdate('M jS', time()));
		$current = $vbulletin->db->query_first("select queries from " . TABLE_PREFIX . "vboptimise where dateline='" . $date . "'");
		$current = intval($current['queries']);
		$add = intval(vb_optimise::$cache->get('vb.optimiser.stats'));
		self::stat(0);
		
		if ($add > 0)
		{
			$vbulletin->db->query("replace into " . TABLE_PREFIX . "vboptimise (dateline, queries) values ('$date', " . ($current + $add) . ")");
		}

		$total = $vbulletin->db->query_first("select sum(queries) as total from " . TABLE_PREFIX . "vboptimise");
		$total = intval($total['total']);

		build_datastore('vbo_resource_savings', $total);
		
		self::report('Added ' . $add . ' count to the statistics.');
	}
	
	

	protected static function timestart()
	{
		if (count(explode(' ', TIMESTART)) > 1)
		{
			list($usec, $sec) = explode(' ', TIMESTART);
			return ((float)$usec + (float)$sec);
		}

		return TIMESTART;
	}

	protected static function saved($saved, $original)
	{
		return (($saved > 0 AND ($original / 100) != 0) ? vb_number_format($saved / ($original / 100), 2) : 0);
	}

	protected static function trimarray(&$array)
	{
		if (is_array($array))
		{
			foreach ($array as $key => $value)
			{
				if (is_array($value))
				{
					self::trimarray($array[$key]);
				}
				else if (is_string($value))
				{
					$array[$key] = trim($value);
				}
			}
		}
	}

	protected static function runtime_scripts()
	{
		static $scripts = false;

		if (!$scripts)
		{
			$scripts = @get_included_files();

			if (!is_array($scripts))
			{
				$scripts = array();
			}

			foreach ($scripts as $key => $value)
			{
				$scripts[$key] = array_pop(explode('/', str_replace('\\', '/', $value)));
			}
		}

		return $scripts;
	}

	/**
	* Output statistics
	*/
	public static function display_stats(&$pageoutput, $mysql_saved = 0, $mysql_total = 0, $php_saved = 0, $php_total = 0, $now = false)
	{
		global $vbulletin, $vbphrase;
		static $vbostats = false;

		if (!$vbulletin->options['vbo_online'])
		{
			return false;
		}

		if (trim($vbulletin->options['vbo_branding_free']) != '')
		{
			if ($vbulletin->options['vbo_branding_free'] == 'eacc7c67b32b55091ef43498f81caf8b')
			{
				return false;
			}
		}

		if (!$vbostats && !$now)
		{
			$vbostats = true;

			$vbulletin->pluginlist['global_complete'] .= "\r\n\r\nvb_optimise::display_stats(\$output, $mysql_saved, $mysql_total, $php_saved, $php_total);";

			if (method_exists(vBulletinHook, 'init'))
			{
				vBulletinHook::init()->set_pluginlist($vbulletin->pluginlist);
			}
			else
			{
				vBulletinHook::set_pluginlist($vbulletin->pluginlist);
			}

			return false;
		}

		$flag = '<!--VBOCR-->';
		$stat = 'vBulletin Optimisation by <a href="http://www.dragonbyte-tech.com/" target="_blank" rel="nofollow">vB Optimise</a> (<!--VBO_SAVED-->).';

		if (strpos($pageoutput, $flag) !== false)
		{
			$pageoutput = str_replace($flag, $flag . '<br />' . $stat, $pageoutput);
		}

		if ($vbulletin->options['vbo_footer_info'])
		{
			$pageoutput = str_replace(' / PHP 0%', '', str_replace('<!--VBO_SAVED-->', 'Reduced on this page: MySQL ' . self::saved($mysql_saved, $mysql_total) . '% / PHP ' . self::saved($php_saved, $php_total) . '%', $pageoutput));
		}
		else
		{
			$pageoutput = str_replace(' (<!--VBO_SAVED-->)', '', $pageoutput);
		}
		
	}
	
	
}